;****************************************************************************
;*
;* 40 x 25 TV Terminal / TV Controller
;*
;* Version 3.0
;*
;*  by Benedikt
;*
;* Email: benedikt83 t gmx.net
;*
;****************************************************************************
;*
;* Erweiterung v2 - von Sven K. (oog)
;*
;*  1) Dynamischer Bildspeicher mit EOL-Zeichen Support
;*  2) TVText Startadresse
;*  3) TVTextDaten im RAM, freie Register fr Hauptprogramm
;*  4) Breite Schrift, 20 Zeichen/Zeile
;*  5) Leerzeilen fr bessere Lesbarkeit
;*  6) Abstand zwischen Zeichen
;*  7) Bildparameter zusammengefasst zu Video-Modes (vmode_n)
;*  8) Grafikmodus
;*  9) Bildzhler
;* 10) Einfache Grafikfunktionen
;*
;*
;* 1) Dynamischer Bildspeicher mit EOL-Zeichen Support
;*
;* Eine Null im Bildspeicher signalisiert das Zeilenende, hnlich wie beim
;* ZX81 mit 1 KB RAM. Das nchste Zeichen wird in neuer Zeile ausgegeben.
;* Der Bildschirm darf grer als der vorhandene Speicher sein, siehe die
;* Demo mit 40*32 Zeichen.
;*
;* Vorteil: Geringer Speicherbedarf
;* Nachteil: Keine fester Bezug von Bildspeicher und Zeichenposition
;*
;* EOL-Modus aktivieren mit:
;* #define eolsupport
;*
;* Zeichencode fr Zeilenende definieren:
;* .equ EOLChar=0
;*
;* Der Code liegt im gleichen Speicher und Timing-Abschnitt wie
;* "#define HOR_FILL", und kann mit diesem nicht zusammen verwendet
;* werden.
;*
;*
;* 2) TVText Startadresse
;*
;* Die Startadresse des Bildschirmspeicher ist frei einstellbar. Zusammen
;* mit dem EOL-Modus kann man beispielsweise einen Text scrollen, der lnger
;* als der Bildschirm ist.
;* Die Variable tvstart(h/l) zeigt auf den Bildspeicher.
;*
;*
;* 3) Im Interrupt genutzte Register werden gesichert.
;* 
;* Alle Register sind frei fr das Hauptprogramm. Eine Integration des
;* Videotreibers in eigene Projekte wird so einfacher.
;*
;*
;* 4) Breite Schrift
;*
;* Die SPI-Clock luft mit halber Geschwindigkeit. Zeichen erscheinen
;* daher doppelt breit
;*
;*
;* 5) Leerzeilen mit ein Pixel hhe
;*
;* Zwischen den Zeilen kann eine beliebige Anzahl Leerzeilen ausgegeben
;* werden. Dies wird ber die Konstante Blanks eingestellt.
;* Leerzeilen erhhen die Lesbarkeit und verringern die CPU-Last, da
;* im Interrupt dann keine Zeichenausgabe erfolgt.
;*
;*
;* 6) Abstand zwischen Zeichen
;*
;* Mit "#define h_blank" wird nach jedem Zeichen ein Fllpixel ausgegeben.
;* So knnen 16 / 32 Zeichen auf voller Bildbreite angezeigt werden
;*
;*
;* 7) Bildparameter
;*
;* Die fr die Darstellung wichtigen Bildparameter sind zusammengefasst
;* und als Parameterstze fr unterschiedliche Video-Modes definiert.
;* Anhand der Beispiele lassen sich leicht eigene Modes erstellen.
;*
;*
;* 8) Grafikmodus
;*
;* Mit "#define gfxmode_n" wird der Grafixmodus aktiviert. Folgende Modes
;* sind implementiert:
;*
;* gfxmode_1 : 80*64 Pixel, zusammenhngend
;* gfxmode_2 : 80*64 Pixel, mit Trennlinie (LCD-hnliches Design)
;*
;* Im Grafikmodus wird von jedem Byte das LSB zuerst ausgegeben, was
;* mit der Hardware-Verschaltung des Videoausgangs an PB1 zusammenhngt.
;* Es gibt eine darauf angepasste Ausgabefunktion (putpixel).
;*
;*
;* 9) Bildzhler
;*
;* Die Variable frames dient als Zhler fr die angezeigten TV-Bilder.
;* Ein Programm kann dies als Zeitbasis fr Animationen oder zur Synchronisation
;* bei der Bildansteuerung verwenden.
;*
;*
;* 10) Einfache Grafikfunktionen
;*
;* Es gibt einige grundlegende Grafikfunktionen.
;*  - putpixel - einen Punkt zeichnen
;*  - hline    - horizontale Linie
;*  - vline    - vertikale Linie
;*  - rect     - Rechteck Auenlinie
;*  - box      - ausgeflltes Rechteck
;*
;* 'color' enthlt die Zeichenfarbe Schwarz, Wei oder Invertiert
;* 
;* Das Demo-Bild zeigt die Anwendung der Grafikfunktionen.
;* Textmode und Grafikmode sind getrennt, so dass nur die jeweils sinnvollen
;* Programmteile eingebunden werden.
;*
;* Deaktiviert man "#define demo", wird aller unntzer Ballast abgeschaltet
;* und es luft nur der Videotreiber.
;*
;****************************************************************************
;*
;* Die Software darf frei kopiert und verndert werden, solange sie nicht
;* ohne meine Erlaubnis fr kommerzielle Zwecke eingesetzt wird.
;*
;****************************************************************************
;*
;* Diese Software erzeugt ein PAL hnliches Videosignal, das von allen von
;* mir getesteten TV Gerten aktzeptiert wird.
;* Somit lassen sich beliebige Texte darstellen. Dank vollstndigem ASCII
;* Zeichensatz sind auch ASCII Grafiken mglich.
;*
;* Das besondere an dieser Software: Das Timing wird per Timer gesteuert,
;* der AVR kann daher nebenbei problemlos andere Aufgaben erledigen, da
;* die Bilderzeugung lediglich etwa 60% der CPU Leistung bentigt.
;* Die Sync Signale werden vollstndig per Timer im PWM Modus erzeugt,
;* die Bildausgabe erfolgt im Interrupt mit automatischer Jitter-
;* kompensation (Idee von Bernhard Schulz).
;* Das Hauptprogramm darf also den Interrupt um bis zu 10 Takte blockieren.
;* Im Hauptprogramm darf jede beliebige Software laufen, solange diese mit
;* den freien Registern und dem wenigen SRAM auskommt.
;*
;* Fuse Bit High: 11001001 = 0XC9 (CKOPT enabled)
;* Fuse Bit Low: 01101110 = 0x6E (externer Quarz, BOD enabled)
;*
;* IO Pins: *
;*
;* PB1 muss mit PB3 verbunden werden
;* PB2 Sync Ausgang: ber 1kOhm an Ausgang
;* PB3 Pixeldaten: ber 330Ohm an Ausgang, Ausgang ber 100Ohm an Masse
;* PB4 DNU (muss frei gelassen werden)
;* PB5 DNU (muss frei gelassen werden)
;* PB6 XTAL 16MHz Quarz
;* PB7 XTAL 16MHz Quarz
;*
;****************************************************************************


.include "m8def.inc"

;.include "m16def.inc"
;#define mega16



;Im Interrupt genutzte und genderte Register:
.def spian   =r16
.def spiaus  =r17
.def X_Cnt   =r18
.def tempi   =r19
;zustzlich: X und Z

;Sonstige Register, fr Hauptprogramm frei:
.def temp    =r20
.def temp2   =r21
.def RX      =r22
.def px1     =r23 ;gfx parameter
.def py1     =r24
.def px2     =r25 ;gfx parameter
.def py2     =r26
.def color   =r27 ;gfx colour
;zustzlich: Y

#ifdef mega16
;PortB
.equ DOUT    =3 ;Pixeldaten \
.equ SS      =4 ;            } Verbinden
.equ MOSI    =5 ;Pixeldaten /
.equ MISO    =6 ;NC
.equ SCK     =7 ;NC

;PortD
.equ Sync    =4 ;Sync Ausgang (OCR1B)

#else
;PortB
.equ DOUT    =1 ;Pixeldaten \
.equ SS      =2 ;            } Verbinden
.equ MOSI    =3 ;Pixeldaten /
.equ MISO    =4 ;NC
.equ SCK     =5 ;NC

;PortB
.equ Sync    =2 ;Sync Ausgang (OCR1B)

#endif

 

.equ Baudrate =250000


#define demo

;Liste der Video-Modes

;#define vmode_0    ;44x34
;#define vmode_1    ;40x32
;#define vmode_2    ;40x25
;#define vmode_3    ;40x25 mit Leerzeile
;#define vmode_4    ;40x20
;#define vmode_5    ;40x16
;#define vmode_6    ;32x24
;#define vmode_7    ;32x24 mit Abstand
;#define vmode_8    ;32x16
;#define vmode_9    ;20x16
;#define vmode_10   ;20x10
;#define vmode_11   ;16x8
;#define gfxmode_1  ;80x64
;#define gfxmode_2  ;80x64 LCD Pixel Modus
;#define gfxmode_3  ;88x72
#define gfxmode_4  ;88x72 LCD Pixel Modus
;#define gfxmode_5  ;96x80
;#define gfxmode_6  ;96x80 LCD Pixel Modus


;#define LCD        ;Zeichen 128-135: Balkenanzeige
;#define HOR_FILL   ;Lcken zwischen den Zeichen fllen
#define eolsupport ;Nicht zusammen mit HOR_FILL setzen
.equ EOLChar=0     ;Zeichencode fr Zeilenende
.equ tvmem=1000    ;Gre fr Bildspeicher, wenn eolsupport gesetzt ist


                   ;Farben fr gfxmode
.equ black  =0     ;Hintergrund
.equ white  =255   ;Vordergrund
.equ invert =254   ;Invertieren


; Video Mode 0: (44*34) eolsupport mit bergroem Bildschirm
;                       Max. Gre fr LCD-TV

#ifdef vmode_0
.equ XSize =44     ;Bildgre in X Richtung in Nibbles
.equ YSize =34     ;Bildgre in Y Richtung pro Halbbild
.equ Height =8     ;Zeichenhhe 
.equ Blanks =0     ;Leerzeilen zwischen Zeilen
;#define stretch_x  ;Pixel doppelt breit, halbe Zeichenzahl/Zeile
;#define h_blank    ;Abstand zwischen Zeichen
.equ XStart =161   ;Anzahl an CPU Takten zwischen Beginn HSync und Bild-
                   ;beginn (mindestens 30, und mindestens HSyncbreite)
.equ BPSize =26    ;VSync Back Porch in Zeilen (Y Verschiebung)
#endif


; Video Mode 1: (40*32) eolsupport mit bergroem Bildschirm

#ifdef vmode_1
.equ XSize =40     ;Bildgre in X Richtung in Nibbles
.equ YSize =32     ;Bildgre in Y Richtung pro Halbbild
.equ Height =8     ;Zeichenhhe 
.equ Blanks =0     ;Leerzeilen zwischen Zeilen
;#define stretch_x  ;Pixel doppelt breit, halbe Zeichenzahl/Zeile
;#define h_blank    ;Abstand zwischen Zeichen
.equ XStart =200   ;Anzahl an CPU Takten zwischen Beginn HSync und Bild-
                   ;beginn (mindestens 30, und mindestens HSyncbreite)
.equ BPSize =32    ;VSync Back Porch in Zeilen (Y Verschiebung)
#endif


; Video Mode 2: (40*25) original tvtext modus

#ifdef vmode_2
.equ XSize =40     ;Bildgre in X Richtung in Nibbles
.equ YSize =25     ;Bildgre in Y Richtung pro Halbbild
.equ Height =8     ;Zeichenhhe 
.equ Blanks =0     ;Leerzeilen zwischen Zeilen
;#define stretch_x  ;Pixel doppelt breit, halbe Zeichenzahl/Zeile
;#define h_blank    ;Abstand zwischen Zeichen
.equ XStart =200   ;Anzahl an CPU Takten zwischen Beginn HSync und Bild-
                   ;beginn (mindestens 30, und mindestens HSyncbreite)
.equ BPSize =54    ;VSync Back Porch in Zeilen (Y Verschiebung)
#endif


; Video Mode 3: (40*25) Eine Leerzeile

#ifdef vmode_3
.equ XSize =40     ;Bildgre in X Richtung in Nibbles
.equ YSize =25     ;Bildgre in Y Richtung pro Halbbild
.equ Height =8     ;Zeichenhhe 
.equ Blanks =1     ;Leerzeilen zwischen Zeilen
;#define stretch_x  ;Pixel doppelt breit, halbe Zeichenzahl/Zeile
;#define h_blank    ;Abstand zwischen Zeichen
.equ XStart =200   ;Anzahl an CPU Takten zwischen Beginn HSync und Bild-
                   ;beginn (mindestens 30, und mindestens HSyncbreite)
.equ BPSize =42    ;VSync Back Porch in Zeilen (Y Verschiebung)
#endif


; Video Mode 4: (40*20) mit Leerzeilen

#ifdef vmode_4
.equ XSize =40     ;Bildgre in X Richtung in Nibbles
.equ YSize =20     ;Bildgre in Y Richtung pro Halbbild
.equ Height =8     ;Zeichenhhe 
.equ Blanks =3     ;Leerzeilen zwischen Zeilen
;#define stretch_x  ;Pixel doppelt breit, halbe Zeichenzahl/Zeile
;#define h_blank    ;Abstand zwischen Zeichen
.equ XStart =200   ;Anzahl an CPU Takten zwischen Beginn HSync und Bild-
                   ;beginn (mindestens 30, und mindestens HSyncbreite)
.equ BPSize =48    ;VSync Back Porch in Zeilen (Y Verschiebung)
#endif


; Video Mode 5: (40*16) mit Leerzeilen

#ifdef vmode_5
.equ XSize =40     ;Bildgre in X Richtung in Nibbles
.equ YSize =16     ;Bildgre in Y Richtung pro Halbbild
.equ Height =8     ;Zeichenhhe 
.equ Blanks =5     ;Leerzeilen zwischen Zeilen
;#define stretch_x  ;Pixel doppelt breit, halbe Zeichenzahl/Zeile
;#define h_blank    ;Abstand zwischen Zeichen
.equ XStart =200   ;Anzahl an CPU Takten zwischen Beginn HSync und Bild-
                   ;beginn (mindestens 30, und mindestens HSyncbreite)
.equ BPSize =50    ;VSync Back Porch in Zeilen (Y Verschiebung)
#endif


; Video Mode 6: (32*24) mit Leerzeilen

#ifdef vmode_6
.equ XSize =32     ;Bildgre in X Richtung in Nibbles
.equ YSize =24     ;Bildgre in Y Richtung pro Halbbild
.equ Height =8     ;Zeichenhhe 
.equ Blanks =0     ;Leerzeilen zwischen Zeilen
;#define stretch_x  ;Pixel doppelt breit, halbe Zeichenzahl/Zeile
;#define h_blank    ;Abstand zwischen Zeichen
.equ XStart =260   ;Anzahl an CPU Takten zwischen Beginn HSync und Bild-
                   ;beginn (mindestens 30, und mindestens HSyncbreite)
.equ BPSize =54    ;VSync Back Porch in Zeilen (Y Verschiebung)
#endif


; Video Mode 7: (32*24) mit Leerzeilen und Zeichenabstand

#ifdef vmode_7
.equ XSize =32     ;Bildgre in X Richtung in Nibbles
.equ YSize =24     ;Bildgre in Y Richtung pro Halbbild
.equ Height =8     ;Zeichenhhe 
.equ Blanks =1     ;Leerzeilen zwischen Zeilen
;#define stretch_x  ;Pixel doppelt breit, halbe Zeichenzahl/Zeile
#define h_blank    ;Abstand zwischen Zeichen
.equ XStart =200   ;Anzahl an CPU Takten zwischen Beginn HSync und Bild-
                   ;beginn (mindestens 30, und mindestens HSyncbreite)
.equ BPSize =44    ;VSync Back Porch in Zeilen (Y Verschiebung)
#endif


; Video Mode 8: (32*16)

#ifdef vmode_8
.equ XSize =32     ;Bildgre in X Richtung in Nibbles
.equ YSize =16     ;Bildgre in Y Richtung pro Halbbild
.equ Height =8     ;Zeichenhhe 
.equ Blanks =5     ;Leerzeilen zwischen Zeilen
;#define stretch_x  ;Pixel doppelt breit, halbe Zeichenzahl/Zeile
#define h_blank    ;Abstand zwischen Zeichen
.equ XStart =200   ;Anzahl an CPU Takten zwischen Beginn HSync und Bild-
                   ;beginn (mindestens 30, und mindestens HSyncbreite)
.equ BPSize =50    ;VSync Back Porch in Zeilen (Y Verschiebung)
#endif


; Video Mode 9: (20*16) stretch_x modus

#ifdef vmode_9
.equ XSize =20     ;Bildgre in X Richtung in Nibbles
.equ YSize =16     ;Bildgre in Y Richtung pro Halbbild
.equ Height =8     ;Zeichenhhe 
.equ Blanks =5     ;Leerzeilen zwischen Zeilen
#define stretch_x  ;Pixel doppelt breit, halbe Zeichenzahl/Zeile
;#define h_blank    ;Abstand zwischen Zeichen
.equ XStart =200   ;Anzahl an CPU Takten zwischen Beginn HSync und Bild-
                   ;beginn (mindestens 30, und mindestens HSyncbreite)
.equ BPSize =50    ;VSync Back Porch in Zeilen (Y Verschiebung)
#endif


; Video Mode 10: (20*10)

#ifdef vmode_10
.equ XSize =20     ;Bildgre in X Richtung in Nibbles
.equ YSize =10     ;Bildgre in Y Richtung pro Halbbild
.equ Height =8     ;Zeichenhhe 
.equ Blanks =12    ;Leerzeilen zwischen Zeilen
#define stretch_x  ;Pixel doppelt breit, halbe Zeichenzahl/Zeile
;#define h_blank    ;Abstand zwischen Zeichen
.equ XStart =200   ;Anzahl an CPU Takten zwischen Beginn HSync und Bild-
                   ;beginn (mindestens 30, und mindestens HSyncbreite)
.equ BPSize =50    ;VSync Back Porch in Zeilen (Y Verschiebung)
#endif


; Video Mode 11: (16*8) Kombination von stretch_x und h_blank

#ifdef vmode_11
.equ XSize =16     ;Bildgre in X Richtung in Nibbles
.equ YSize =8      ;Bildgre in Y Richtung pro Halbbild
.equ Height =8     ;Zeichenhhe 
.equ Blanks =20    ;Leerzeilen zwischen Zeilen
#define stretch_x  ;Pixel doppelt breit, halbe Zeichenzahl/Zeile
#define h_blank    ;Abstand zwischen Zeichen
.equ XStart =200   ;Anzahl an CPU Takten zwischen Beginn HSync und Bild-
                   ;beginn (mindestens 30, und mindestens HSyncbreite)
.equ BPSize =50    ;VSync Back Porch in Zeilen (Y Verschiebung)
#endif


#ifdef gfxmode_1
#define gfxmode
#define gfx_w1
.equ XSize  =10    ;Anzahl Bytes/Zeile
.equ YSize  =64    ;Anzahl Zeilen
.equ Height =4     ;Zelenhhe
.equ Blanks =0     ;Leerzeilen zwischen Zeilen
.equ XStart =160   ;Anzahl an CPU Takten zwischen Beginn HSync und Bild-
                   ;beginn (mindestens 30, und mindestens HSyncbreite)
.equ BPSize =32    ;VSync Back Porch in Zeilen (Y Verschiebung)
#endif


#ifdef gfxmode_2
#define gfxmode
#define gfx_w1
#define gfxpix
.equ XSize  =10    ;Anzahl Bytes/Zeile
.equ YSize  =64    ;Anzahl Zeilen
.equ Height =3     ;Zelenhhe
.equ Blanks =1     ;Leerzeilen zwischen Zeilen
.equ XStart =160   ;Anzahl an CPU Takten zwischen Beginn HSync und Bild-
                   ;beginn (mindestens 30, und mindestens HSyncbreite)
.equ BPSize =32    ;VSync Back Porch in Zeilen (Y Verschiebung)
#endif

#ifdef gfxmode_3
#define gfxmode
.equ XSize  =11    ;Anzahl Bytes/Zeile
.equ YSize  =72    ;Anzahl Zeilen
.equ Height =3     ;Zelenhhe
.equ Blanks =0     ;Leerzeilen zwischen Zeilen
.equ XStart =200   ;Anzahl an CPU Takten zwischen Beginn HSync und Bild-
                   ;beginn (mindestens 30, und mindestens HSyncbreite)
.equ BPSize =52    ;VSync Back Porch in Zeilen (Y Verschiebung)
#endif


#ifdef gfxmode_4
#define gfxmode
#define gfxpix
.equ XSize  =11    ;Anzahl Bytes/Zeile
.equ YSize  =72    ;Anzahl Zeilen
.equ Height =2     ;Zelenhhe
.equ Blanks =1     ;Leerzeilen zwischen Zeilen
.equ XStart =200   ;Anzahl an CPU Takten zwischen Beginn HSync und Bild-
                   ;beginn (mindestens 30, und mindestens HSyncbreite)
.equ BPSize =52    ;VSync Back Porch in Zeilen (Y Verschiebung)
#endif

#ifdef gfxmode_5
#define gfxmode
.equ XSize  =12    ;Anzahl Bytes/Zeile
.equ YSize  =80    ;Anzahl Zeilen
.equ Height =3     ;Zelenhhe
.equ Blanks =0     ;Leerzeilen zwischen Zeilen
.equ XStart =160   ;Anzahl an CPU Takten zwischen Beginn HSync und Bild-
                   ;beginn (mindestens 30, und mindestens HSyncbreite)
.equ BPSize =44    ;VSync Back Porch in Zeilen (Y Verschiebung)
#endif


#ifdef gfxmode_6
#define gfxmode
#define gfxpix
.equ XSize  =12    ;Anzahl Bytes/Zeile
.equ YSize  =80    ;Anzahl Zeilen
.equ Height =2     ;Zelenhhe
.equ Blanks =1     ;Leerzeilen zwischen Zeilen
.equ XStart =160   ;Anzahl an CPU Takten zwischen Beginn HSync und Bild-
                   ;beginn (mindestens 30, und mindestens HSyncbreite)
.equ BPSize =44    ;VSync Back Porch in Zeilen (Y Verschiebung)
#endif


;Horizontal
.equ SyncWidth =75 ;Anzahl an CPU Takten fr HSync

;Vertikal
.equ VSize =4      ;VSync Impuls in Zeilen



;Interne Konstanten
.equ BildSize =YSize*(Height+Blanks) ;Bildhhe in Zeilen
.equ BildStart =313-BildSize+YSize   ;Bildhhe in SyncZeilen+Zeichen 
                                     ;(interner Zhler)
.equ FPSize =(313-YSize*(Height+Blanks))-VSize-BPSize
                              ;Frontporchbreite in Zeilen
.equ FPStart =YSize           ;Frontporch Start in internen Zhlerzeilen
.equ VSStart =FPStart+FPSize  ;Vsyncstart in internen Zhlerzeilen
.equ BPStart =VSStart+VSize   ;Backporchstart in internen Zhlerzeilen 


.dseg

	;Sicherung der im Interrupt genderten Register

s_xh:      .byte 1
s_xl:      .byte 1
s_zh:      .byte 1
s_zl:      .byte 1
s_zh_line: .byte 1
s_sreg:    .byte 1
s_X_Cnt:   .byte 1
s_tempi:   .byte 1

	;Daten der Interrupt-Routine

m_Y_Cnt:   .byte 1		
m_C_Cnt:   .byte 1
m_tvptrh:  .byte 1	;Zeiger auf auszugebendes Zeichen (xh/xl)
m_tvptrl:  .byte 1
tvstarth:  .byte 1	;Bildspeicher Startadresse
tvstartl:  .byte 1
frame:     .byte 1  ;Zhlt TV-Bilder (50/Sekunde)


#ifdef gfxmode
DDRAM: .byte XSize*YSize      ;DDRAM
#else ;gfxmode

#ifdef eolsupport
DDRAM: .byte tvmem            ;DDRAM
#else
DDRAM: .byte XSize*YSize      ;DDRAM
#endif

#endif ;gfxmode


	;*********************************************
	; Makros
	;*********************************************

.macro nops                ;Pixelbreite mit NOP vergrern
#ifdef gfx_w1
	nop
	;nop
	;nop
#else ;gfx_w1
	;nop
	;nop
#endif ;gfx_w1
.endmacro

.macro lnops               ;Pixelausgleich, letztes Pixel
#ifdef gfx_w1
	nop
#endif ;gfx_w1
;	nop
	nop
	nop
#ifdef gfxpix
	cbi   PORTB, DOUT      ;Bild: schwarz (gfxmode LCD Design)
#else
	nop
	nop
#endif
	nop
.endmacro

.macro dnops               ;Pixelausgleich zwischen Pixeln
	nop
	lnops
.endmacro


;****************************************************************************
; Interrupt Vektoren
;****************************************************************************

.cseg
.org 0
	rjmp  Reset

.org OC1Aaddr ;Bildbeginn

	;******************************************
	; register save - part 1 - critical timing
	;******************************************

	sts   s_tempi,tempi
	in    tempi, sreg
	sts   s_sreg, tempi
	sts   s_zh,zh
	sts   s_zl,zl
	sts   s_X_Cnt,X_Cnt

	lds   tempi,m_Y_Cnt
	cpi   tempi,YSize ;Bildbereich oder schon VSync + Zubehr ?
	brlo  lineout
	rjmp  handle_VSync

lineout:

	in    ZL, TCNT1L ;Timerwert lesen
	in    ZH, TCNT1H

	subi  ZL, low(XStart+10-comp) ;und Korrekturwert berechnen
	sbci  ZH, high(XStart+10-comp)

	ijmp  ;Wartezeit korrigieren
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop


	;*********************************************
	; register save - part 2a - noncritical timing
	;*********************************************

	sts   s_xh,xh       ; save xh/xl
	sts   s_xl,xl
	lds   xh,m_tvptrh   ; load pointer to actual character into xh/xl
	lds   xl,m_tvptrl



	;*********************************************
	; Eine Bildzeile im Grafikmodus ausgeben
	;*********************************************

#ifdef gfxmode

comp:
	lds   ZH,s_zh_line

	lds   tempi,m_C_Cnt    ;insert optionally blank lines
	cpi   tempi,Blanks+1
	brsh  gfxstart
	rjmp  skiphloop

gfxstart:

	ldi   tempi, XSize
	mov   X_Cnt, tempi
hloop:                     ;Loop fr 1 Zeile
	ld    tempi, X+        ;2 ASCII aus RAM lesen 2
#ifdef gfxpix
	cbi   PORTB, DOUT      ;Bild: schwarz (gfxmode LCD Design)
#else
	nop
	nop
#endif
	rol   tempi            ;1
	out   PORTB, tempi     ;1 Bit 0
	ror   tempi            ;1
	nops
	dnops
	out   PORTB, tempi     ;1 Bit 1
	ror   tempi            ;1
	nops
	dnops
	out   PORTB, tempi     ;1 Bit 2
	ror   tempi            ;1
	nops
	dnops
	out   PORTB, tempi     ;1 Bit 3
	ror   tempi            ;1
	nops
	dnops
	out   PORTB, tempi     ;1 Bit 4
	ror   tempi            ;1
	nops
	dnops
	out   PORTB, tempi     ;1 Bit 5
	ror   tempi            ;1
	nops
	dnops
	out   PORTB, tempi     ;1 Bit 6
	ror   tempi            ;1
	dec   X_Cnt            ;1
	nops
	lnops
	out   PORTB, tempi     ;1 Bit 7
	nops

#ifdef gfx_w1
	breq  gfxend           ;1/2
	rjmp  hloop            ;2
#else ;gfx_w1
	brne  hloop
#endif ;gfx_w1



gfxend:
	nop                    ;1 Letztes Pixel zeichnen
	nop                    ;1
	nop                    ;1
#ifndef gfxpix
	nop                    ;1
	nop                    ;1
#endif

skiphloop:
	cbi   PORTB, DOUT      ;Bild: schwarz




	;*********************************************
	; Eine Bildzeile im Textmodus ausgeben
	;*********************************************

#else ;gfxmode

	push  spian
	push  spiaus
#ifdef mega16
	ldi   spian, (1<<MOSI)|(1<<SS)
	ldi   spiaus, (1<<DOUT)|(1<<SS)
#else
	ldi   spian, (1<<Sync)|(1<<MOSI)|(1<<SS)
	ldi   spiaus, (1<<Sync)|(1<<DOUT)|(1<<SS)
#endif


comp:
	lds   ZH,s_zh_line

	lds   tempi,m_C_Cnt    ; insert optionally blank lines
	cpi   tempi,Blanks+1
	brlo  skiphloop

	ldi   tempi, XSize
	mov   X_Cnt, tempi
hloop:                     ;Loop fr 1 Zeile
	ld    ZL, X+           ;ASCII aus RAM lesen 2
	lpm   tempi, Z         ;Byte aus CGROM lesen 3
	out   DDRB, spiaus     ;SPI aus, Port an 1
	out   SPDR, tempi      ;Byte ausgeben 1
	out   DDRB, spian      ;SPI an, Port aus 1

#ifdef eolsupport
	cpi   ZL,EOLChar	   ; 1    if character is EOLChar
	brne  noteol	       ; 1/2  ..
	ldi   X_Cnt,1	       ; 1    then set counter to last character
noteol:
	nop                    ; 1
	nop                    ; 1

#else
#ifdef HOR_FILL
	sbrs  tempi, 0         ;Fllpixel setzen 1
	cbi   PORTB, DOUT      ;1,5
	sbrc  tempi, 0         ;1
	sbi   PORTB, DOUT      ;1,5

#else
	nop                    ; 1
	nop                    ; 1
	nop                    ; 1
	nop                    ; 1
	nop                    ; 1
#endif


#endif


#ifdef h_blank
	nop                    ; 1     h-blank
	nop                    ; 1     h-blank
	nop                    ; 1     h-blank
	nop                    ; 1     h-blank
#endif


#ifdef stretch_x
	nop                    ; 1 waste 18 clock cycles
	nop                    ; 1
	nop                    ; 1
	nop                    ; 1
	nop                    ; 1
	nop                    ; 1
	nop                    ; 1
	nop                    ; 1
	nop                    ; 1
	nop                    ; 1
	nop                    ; 1
	nop                    ; 1
	nop                    ; 1
	nop                    ; 1
	nop                    ; 1
	nop                    ; 1
	nop                    ; 1
	nop                    ; 1

#ifdef h_blank
	nop                    ; 1     h-blank
	nop                    ; 1     h-blank
	nop                    ; 1     h-blank
	nop                    ; 1     h-blank
#endif

#endif

	nop                    ; 1
	nop                    ; 1
	dec   X_Cnt            ; 1
	brne  hloop            ; 2
                           ; =18, also 16 (fr 1 Byte) + Pause dazwischen
	nop
	nop
	nop
	nop

skiphloop:

	out   DDRB, spiaus     ;SPI abschalten
	cbi   PORTB, DOUT      ;Bild: schwarz


	pop  spiaus
	pop  spian


#endif ;gfxmode

	;*********************************************
	; Ausgabe einer Bildzeile - Ende
	;*********************************************


	lds   tempi,m_C_Cnt
	dec   tempi            ;Zhler fr Zeichenhhe erniedrigen
	sts   m_C_Cnt,tempi

	breq skipcloop         ;1 Textzeile fertig ausgegeben ?
	
	inc ZH                 ;Nein -> nchste Pixelzeile

	cpi   tempi,Blanks+1
	brlo  blankskip        ;Keine Adressnderung bei Leerzeilen

	lds   XL,m_tvptrl	   ;Textadresse wieder an Anfang der Textzeile setzen
	lds   XH,m_tvptrh

blankskip:

	rjmp  exit_int
	
skipcloop:                 ;Alles fr neue Textzeile einrichten
	ldi   ZH, 4            ;1024Bytes CGROM Offset

	ldi   tempi, Height+Blanks ;Zeichenhhen Counter laden
	sts   m_C_Cnt,tempi
	lds   tempi,m_Y_Cnt	   ;Zeilencounter erhhen
	inc   tempi
	sts   m_Y_Cnt,tempi

	rjmp  exit_int



handle_VSync:

	;*********************************************
	; register save - part 2b - noncritical timing
	;*********************************************

	sts   s_xh,xh
	sts   s_xl,xl
	lds   ZH,s_zh_line

	inc   tempi             ;Zeilenzhler erhhen
	sts   m_Y_Cnt,tempi

	cpi   tempi, VSStart    ;Beginn VSync ?
	breq  VSyncstart 

	cpi   tempi, BPStart    ;Ende VSync ?
	breq  VSyncende 

	cpi   tempi, BildStart  ;Bildende ?
	brsh  Bildbeginn 

	rjmp  exit_int

VSyncstart:
	ldi   tempi, (1<<COM1B1)|(0<<COM1B0)|(1<<WGM11)|(1<<WGM10)
	out   TCCR1A, tempi     ;VSync Start: Sync Impulse High aktiv
	rjmp  exit_int

VSyncende:
	ldi   tempi, (1<<COM1B1)|(1<<COM1B0)|(1<<WGM11)|(1<<WGM10)
	out   TCCR1A, tempi     ;VSync Ende: Sync Impulse Low aktiv
	rjmp  exit_int

Bildbeginn:
	lds   tempi, frame
	inc   tempi
	sts   frame, tempi

	clr   tempi
	sts   m_Y_Cnt,tempi

	lds   xh,tvstarth		;Zeiger auf Bildspeicher auf start setzen
	lds   xl,tvstartl
	sts   m_tvptrh,xh
	sts   m_tvptrl,xl

	rjmp  exit_int

exit_int:

	sts   m_tvptrh,xh       ;Gesicherte Werte der Register zurckladen
	sts   m_tvptrl,xl
	lds   X_Cnt,s_X_Cnt
	sts   s_zh_line,ZH
	lds   zh,s_zh
	lds   zl,s_zl
	lds   xh,s_xh
	lds   xl,s_xl
	lds   tempi,s_sreg
	out   sreg,tempi
	lds   tempi,s_tempi

	reti
 




;****************************************************************************
; CPU Register Initialisierung
;****************************************************************************

Reset:

	ldi   temp, low(RAMEND)
	out   SPL, temp         ;setup stack pointer

	ldi   temp, high(RAMEND)
	out   SPH, temp         ;setup stack pointer

	ser   temp
	out   portd, temp

	clr   temp
	out   portb, temp

#ifdef mega16

#ifdef gfxmode
	ldi   temp, (1<<DOUT)|(0<<MOSI)|(1<<SS)
#else ;gfxmode
	ldi   temp, (1<<MOSI)|(1<<SS)
#endif ;gfxmode

	out   DDRB, temp
	sbi   DDRD, Sync

#else ;mega16

#ifdef gfxmode
	ldi   temp, (1<<DOUT)|(1<<Sync)|(0<<MOSI)|(1<<SS)
#else ;gfxmode
	ldi   temp, (1<<Sync)|(1<<MOSI)|(1<<SS)
#endif ;gfxmode

	out   DDRB, temp

#endif ;mega16

	cbi   PORTB, DOUT
	ldi   temp, (1<<SPE)|(1<<MSTR)|(1<<CPHA) ;fclk/2 Schiebetakt
	out   SPCR, temp        ;Enable SPI Slave, Interrupt

#ifdef stretch_x
	cbi   SPSR, SPI2X
#else
	sbi   SPSR, SPI2X
#endif

	ldi   temp, high(1024)  ;Timer Reloadwert
	out   ICR1H, temp

	ldi   temp, low(1024)   ;Timer Reloadwert
	out   ICR1L, temp

	ldi   temp, high(SyncWidth) ;HSync Breite in CPU Takten
	out   OCR1BH, temp

	ldi   temp, low(SyncWidth)  ;HSync Breite in CPU Takten
	out   OCR1BL, temp

	ldi   temp, high(XStart-30) ;H Shift
	out   OCR1AH, temp

	ldi   temp, low(XStart-30)  ;H Shift
	out   OCR1AL, temp

	ldi   temp, (1<<COM1B1)|(1<<COM1B0)|(1<<WGM11)|(1<<WGM10)
	out   TCCR1A, temp

	ldi   temp, (1<<WGM12)|1 ;10bit PWM bei 16MHz: 15,625kHz
	out   TCCR1B, temp

	ldi   temp, (1<<OCIE1A)  ;CompareA Interrupt: Beginn der Datenausgabe
	out   timsk, temp

	ldi   temp, 0
	out   UBRRH, temp

	ldi   temp, (1000000+Baudrate/2)/Baudrate-1;Baudrate bei 16MHz berchnen und runden
	out   UBRRL, temp
	ldi   temp, (1<<RXEN)    ;RX aktivieren
	out   UCSRB, temp 



;****************************************************************************
; Variablen Initialisierung
;****************************************************************************

	clr   temp
	clr   temp2


; startbild

	ldi   YL, low(DDRAM)
	ldi   YH, high(DDRAM)

#ifdef demo

#ifdef gfxmode

	;*********************************************
	;* Demo - Startbild (gfxmode)
	;*********************************************

	ldi   temp, 0
clrloop:
	st    Y+, temp
	inc   temp
#ifdef eolsupport
	ldi   temp2, high(tvmem+DDRAM)
	cpi   YL, low(tvmem+DDRAM)
#else
	ldi   temp2, high(XSize*YSize+DDRAM)
	cpi   YL, low(XSize*YSize+DDRAM)
#endif
	cpc   YH, temp2
	brlo  clrloop



	;Rahmen test
	ldi   color,white
	ldi   px1, 0
	ldi   py1, 0
	ldi   px2, XSize*8-1
	ldi   py2, YSize-1
	rcall rect


	;Rahmen test
	ldi   color,black
	ldi   px1, 1
	ldi   py1, 1
	ldi   px2, XSize*8-1-1
	ldi   py2, YSize-1-1
	rcall rect


	;Rahmen test
	ldi   color,white
	ldi   px1, 2
	ldi   py1, 2
	ldi   px2, XSize*8-1-2
	ldi   py2, YSize-1-2
	rcall rect


	;Rahmen test - parameter vertauscht, x1,y2 = unten rechts
	ldi   color,black
	ldi   px2, 3
	ldi   py2, 3
	ldi   px1, XSize*8-1-3
	ldi   py1, YSize-1-3
	rcall rect


	;hline test
	ldi   color,black
	ldi   px1, 4
	ldi   py1, YSize/2
	ldi   px2, XSize*8-1-4
	rcall hline


	;hline test
	ldi   color,white
	ldi   px1, 4
	ldi   py1, YSize/2-1
	ldi   px2, XSize*8-1-4
	rcall hline


	;vline test
	ldi   color,black
	ldi   px1, XSize*8/2
	ldi   py1, 4
	ldi   py2, YSize-1-4
	rcall vline


	;vline test
	ldi   color,white
	ldi   px1, XSize*8/2-1
	ldi   py1, 4
	ldi   py2, YSize-1-4
	rcall vline


	;box test
	ldi   color,black
	ldi   px1, 4
	ldi   py1, 4
	ldi   px2, XSize*8/2-2
	ldi   py2, YSize/2-2
	rcall box


	;box test
	ldi   color,white
	ldi   px1, XSize*8/2+1
	ldi   py1, YSize/2+1
	ldi   px2, XSize*8-5
	ldi   py2, YSize-5
	rcall box


	;diagonal
	ldi   color,white
	ldi   px1, YSize/2-1
	ldi   py1, YSize/2-1
diagonal:
	rcall putpixel
	dec   px1
	dec   py1
	brne  diagonal



#else ;gfxmode

	;*********************************************
	;* Demo - Startbild (textmode)
	;*********************************************

; "TVText" Message

	rcall print
	.db   "TVText-V2 ",0

; Bildgre anzeigen (Zeichen x Zeilen)

	ldi   temp,'0'+XSize/10
	st    Y+, temp
	ldi   temp,'0'+XSize-XSize/10*10
	st    Y+, temp

	ldi   temp,'x'
	st    Y+, temp

	ldi   temp,'0'+YSize/10
	st    Y+, temp
	ldi   temp,'0'+YSize-YSize/10*10
	st    Y+, temp

	ldi   temp,0
	st    Y+, temp


	rcall print
	.db   "AVR Videogenerator",0
	ldi   temp,0
	st    Y+, temp


	rcall print
	.db   "www.mikrocontroller.net",0
	ldi   temp,0
	st    Y+, temp


	ldi   temp,'>'
	st    Y+, temp
	ldi   temp,0
	st    Y+, temp
	ldi   temp,'>'
	st    Y+, temp
	st    Y+, temp
	ldi   temp,0
	st    Y+, temp
	ldi   temp,'>'
	st    Y+, temp
	ldi   temp,0
	st    Y+, temp

; ASCII Codes

	ldi   temp, 32
clrloop:
	st    Y+, temp

	tst   temp         ;Hinter 0 Zeilenumbrche einfgen fr EOLSupport
	brne  nozero
	ldi   xl,0
	ldi   xh,'>'
	st    Y+, xh
	st    Y+, xl
	st    Y+, xh
	st    Y+, xh
	st    Y+, xl
	st    Y+, xh
	st    Y+, xl
nozero:

	inc   temp
#ifdef eolsupport
	ldi   temp2, high(tvmem+DDRAM)
	cpi   YL, low(tvmem+DDRAM)
#else
	ldi   temp2, high(XSize*YSize+DDRAM)
	cpi   YL, low(XSize*YSize+DDRAM)
#endif
	cpc   YH, temp2
	brlo  clrloop

#endif ;gfxmode

#else	  ;#def demo
; ASCII Codes

	ldi   temp, 32
clrloop:
	st    Y+, temp
	inc   temp
#ifdef eolsupport
	ldi   temp2, high(tvmem+DDRAM)
	cpi   YL, low(tvmem+DDRAM)
#else
	ldi   temp2, high(XSize*YSize+DDRAM)
	cpi   YL, low(XSize*YSize+DDRAM)
#endif
	cpc   YH, temp2
	brlo  clrloop


#endif

	ldi   XL, low(DDRAM)
	ldi   XH, high(DDRAM)

	sts   tvstarth,xh		; save tv screen start pointer
	sts   tvstartl,xl
	sts   m_tvptrh,xh
	sts   m_tvptrl,xl



	ldi   temp, (1<<OCF1A) ;Interrupt Flag lschen
	out   TIFR, temp
	sei


;****************************************************************************
; Hauptprogramm
;****************************************************************************

Mainloop:

#ifdef demo

	;*************************************************
	;* Demo start (zum Betrieb nicht notwendig)
	;*************************************************

	;Lsche alle Register im Hauptprogramm
	;Treiber muss trotzdem einwandfrei laufen

	clr   r0
	clr   r1
	clr   r2
	clr   r3
	clr   r4
	clr   r5
	clr   r6
	clr   r7

	clr   r8
	clr   r9
	clr   r10
	clr   r11
	clr   r12
	clr   r13
	clr   r14
	clr   r15

	clr   r16
	clr   r17
	clr   r18
	clr   r19
	clr   r20
	clr   r21
	clr   r22
	clr   r23

	clr   r24
	clr   r25
	clr   r26
	clr   r27
	clr   r28
	clr   r29
	clr   r30
	clr   r31

	;ndere alle Register im Hauptprogramm
	;Treiber muss trotzdem einwandfrei laufen

	dec   r0
	dec   r1
	dec   r2
	dec   r3
	dec   r4
	dec   r5
	dec   r6
	dec   r7

	dec   r8
	dec   r9
	dec   r10
	dec   r11
	dec   r12
	dec   r13
	dec   r14
	dec   r15

	dec   r16
	dec   r17
	dec   r18
	dec   r19
	dec   r20
	dec   r21
	dec   r22
	dec   r23

	dec   r24
	dec   r25
	dec   r26
	dec   r27
	dec   r28
	dec   r29
	dec   r30
	dec   r31

	;Prfe Bildstabilitt bei Interrupt-Sperre

	cli
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	sei

	;Verschiebe die Bildschirm Startadresse mit Tasten an PD2 - PD4
	;(Taster gegen Gnd schalten)


	;Taste A (PD2)

	sbic  pind,2
	rjmp  key_a_end
	;
	ldi   temp,1
	lds   temp2,tvstartl
	sub   temp2,temp
	sts   tvstartl,temp2
	ldi   temp,0
	lds   temp2,tvstarth
	sbc   temp2,temp
	sts   tvstarth,temp2
key_a_wait:
	nop
	nop
	nop
	dec   temp2
	brne  key_a_wait
	dec   temp
	brne  key_a_wait
key_a_end:


	;Taste B (PD3)

	sbic  pind,3
	rjmp  key_b_end
	;
	ldi   temp,1
	lds   temp2,tvstartl
	add   temp2,temp
	sts   tvstartl,temp2
	ldi   temp,0
	lds   temp2,tvstarth
	adc   temp2,temp
	sts   tvstarth,temp2
key_b_wait:
	nop
	nop
	nop
	dec   temp2
	brne  key_a_wait
	dec   temp
	brne  key_a_wait
key_b_end:


	;Taste C (PD4)

	sbic  pind,4
	rjmp  key_c_end

	ldi   temp, low(DDRAM)
	sts   tvstartl,temp
	ldi   temp, high(DDRAM)
	sts   tvstarth,temp
key_c_end:


	;Bildschirmbereich invertieren (fr gfxmode)

#ifdef gfxmode
	;Taste D (PD5)

	sbic  pind,5
	rjmp  key_d_end

	ldi   color,invert
	ldi   px1,4
	ldi   py1,4
	ldi   px2,XSize*8-1-4
	ldi   py2,YSize-1-4
	rcall box
key_d_end:
#endif


;	lds   tempi, frame   ;Frame-Zhler an erster Bildposition speichern
;	lds   yl,tvstartl
;	lds   yh,tvstarth
;	st    Y,tempi


#endif    ;#def demo


	;*************************************************
	;* Demo ende (zum Betrieb nicht notwendig)
	;*************************************************


	sbic  UCSRA, RXC ;Daten im UART Puffer ? 
	rcall rxuart
	rjmp  Mainloop


rxuart: ;UART Daten verarbeiten
	in    RX, UDR
	st    Y+, RX ;Daten
	ldi   temp, high(XSize*YSize+DDRAM)
	cpi   YL, low(XSize*YSize+DDRAM)
	cpc   YH, temp
	brlo  skipwpreset

	ldi   YL, low(DDRAM)
	ldi   YH, high(DDRAM)
skipwpreset:
	ret


	;*************************************************
	;* Subroutine start (zum Betrieb nicht notwendig)
	;*************************************************

#ifdef demo

#ifdef gfxmode


	;*************************************************
	;* putpixel - einen Punkt zeichnen oder lschen
	;* color = Pixelfarbe
	;* px1   = X-Koordinate
	;* py1   = Y-Koordinate
	;*************************************************

putpixel:
	cpi   px1,XSize*8
	brsh  putpix_err
	cpi   py1,YSize
	brsh  putpix_err

	push  temp
	lds   yl,tvstartl
	lds   yh,tvstarth

	;Berechne Adresse des Bytes im Bildspeicher

	;Y=Y+px
	mov   temp,px1
	lsr   temp
	lsr   temp
	lsr   temp
	add   yl,temp
	clr   temp
	adc   yh,temp

	;Y=Y+py*XSize (brute force)

	push  r0
	push  r1
	ldi   temp,XSize
	mul   py1,temp
	add   yl,r0
	adc   yh,r1
	pop   r1
	pop   r0

	push  px1
	push  zh
	push  zl

	;Lade Bit des Pixel

	ldi   zh,high(putpixtab*2)
	ldi   zl,low(putpixtab*2)
	andi  px1,7
	add   zl,px1
	clr   px1
	adc   zh,px1
	lpm   temp,Z

	;Teste Farbe (color)

	cpi   color,white
	brne  putpux_notwhite

	;Setze Pixel im Bildspeicher

	ld    px1,Y
	or    temp,px1
	st    Y,temp
	rjmp  putpix_end

putpux_notwhite:

	cpi   color,black
	brne  putpux_notblack

	;Lsche Pixel im Bildspeicher

	ld    px1,Y
	com   temp
	and   temp,px1
	st    Y,temp
	rjmp  putpix_end

putpux_notblack:

	;Invertiere Pixel im Bildspeicher

	ld    px1,Y
	eor   temp,px1
	st    Y,temp

putpix_end:

	pop   zl
	pop   zh
	pop   px1

	pop   temp
putpix_err:
	ret

putpixtab:
	.db 1,2,4,8,16,32,64,128


	;*************************************************
	;* rect - ein Rechteck zeichnen, nur den Rahmen
	;* color = Pixelfarbe
	;* px1   = X-Koordinate Anfang
	;* py1   = Y-Koordinate Anfang
	;* px2   = X-Koordinate Ende
	;* py2   = Y-Koordinate Ende
	;*************************************************

rect:
	push  px2
	push  py2
	push  px1
	push  py1


	;Koordinaten sortieren
	;x1<x2 und y1<y2

	cp    px1,px2
	brlo  rect_x_ok
	breq  rect_e
	
	eor   px1,px2            ;Tausche px1, px2
	eor   px2,px1
	eor   px1,px2
rect_x_ok:

	cp    py1,py2
	brlo  rect_y_ok
	breq  rect_e
	
	eor   py1,py2            ;Tausche py1, py2
	eor   py2,py1
	eor   py1,py2
rect_y_ok:

	push  px1
	push  py1

rect_l1:
	rcall putpixel
	inc   px1
	cp    px1,px2
	brne  rect_l1

rect_l2:
	rcall putpixel
	inc   py1
	cp    py1,py2
	brne  rect_l2

	pop   py2
	pop   px2

rect_l3:
	rcall putpixel
	dec   px1
	cp    px1,px2
	brne  rect_l3

rect_l4:
	rcall putpixel
	dec   py1
	cp    py1,py2
	brne  rect_l4

rect_e:
	pop   py1
	pop   px1
	pop   py2
	pop   px2
	ret


	;*************************************************
	;* hline - eine horizontale Linie zeichnen
	;* color = Pixelfarbe
	;* px1   = X-Koordinate Anfang
	;* py1   = Y-Koordinate Anfang
	;* px2   = X-Koordinate Ende
	;*************************************************

hline:
	push  px1
	push  px2

	;Koordinaten sortieren
	;x1<x2

	cp    px1,px2
	brlo  hline_x_ok
	breq  hline_e
	
	eor   px1,px2            ;Tausche px1, px2
	eor   px2,px1
	eor   px1,px2
hline_x_ok:

	rcall putpixel
hline_l:
	inc   px1
	rcall putpixel
	cp    px1,px2
	brne  hline_l

hline_e:
	pop   px2
	pop   px1
	ret


	;*************************************************
	;* vline - eine vertikale Linie zeichnen
	;* color = Pixelfarbe
	;* px1   = X-Koordinate Anfang
	;* py1   = Y-Koordinate Anfang
	;* py2   = Y-Koordinate Ende
	;*************************************************

vline:
	push  py1
	push  py2

	;Koordinaten sortieren
	;y1<y2

	cp    py1,py2
	brlo  vline_y_ok
	breq  vline_e
	
	eor   py1,py2            ;Tausche py1, py2
	eor   py2,py1
	eor   py1,py2
vline_y_ok:

	rcall putpixel
vline_l:
	inc   py1
	rcall putpixel
	cp    py1,py2
	brne  vline_l

vline_e:
	pop   py2
	pop   py1
	ret



	;*************************************************
	;* box - ein ausgeflltes Rechteck zeichnen
	;* color = Pixelfarbe
	;* px1   = X-Koordinate Anfang
	;* py1   = Y-Koordinate Anfang
	;* px2   = X-Koordinate Ende
	;* py2   = Y-Koordinate Ende
	;*************************************************

box:
	push  px2
	push  py2
	push  px1
	push  py1


	;Koordinaten sortieren
	;x1<x2 und y1<y2

	cp    px1,px2
	brlo  box_x_ok
	breq  box_e
	
	eor   px1,px2            ;Tausche px1, px2
	eor   px2,px1
	eor   px1,px2
box_x_ok:

	cp    py1,py2
	brlo  box_y_ok
	breq  box_e
	
	eor   py1,py2            ;Tausche py1, py2
	eor   py2,py1
	eor   py1,py2
box_y_ok:

	rcall hline
box_l:
	inc   py1
	rcall hline
	cp    py1,py2
	brne  box_l

box_e:
	pop   py1
	pop   px1
	pop   py2
	pop   px2
	ret




#else ;gfxmode



	;*************************************************
	;* print - einen Text ausgeben
	;* YH:YL = Zeiger auf Bildschirmspeicher (*)
	;*
	;* Der Text steht hinter dem rcall-Befehl im
	;* Programm und endet mit einer Null.
	;*
	;* Beispiel
	;*  rcall print
	;*  .db "Hello World",0
	;*
	;* (*) Der Zeiger YH:YL zeigt anschlieend auf
	;* die nchste Position hinter dem Text
	;*
	;* Keine Prfung auf Speicherberlauf
	;*
	;*************************************************

print:
	pop   zh
	pop   zl
	
	lsl   zl
	rol   zh

print_l:
	lpm   temp,z+
	tst   temp
	breq  print_e

	st    y+,temp     ;Zeichen in Bildspeicher schreiben
	rjmp  print_l

print_e:
    inc   zl          ;Return-Adresse berechnen
	adc   zh,temp     ;temp ist Null an dieser Stelle
	lsr   zh
	ror   zl
	ijmp              ;Und mit einem indirekten Sprung zurck

#endif ;gfxmode
#endif ;demo

	;*************************************************
	;* Subroutine ende (zum Betrieb nicht notwendig)
	;*************************************************






#ifdef gfxmode

#else ;gfxmode


;*** CG ROM Daten *** (8x8 Zeichensatz)

;
; Format der Daten
;
; Zeile 0: 0x30  ..##....
; Zeile 1: 0x78  .####...
; Zeile 2: 0xcc  ##..##..
; Zeile 3: 0xcc  ##..##..
; Zeile 4: 0xfc  ######..
; Zeile 5: 0xcc  ##..##..
; Zeile 6: 0xcc  ##..##..
; Zeile 7: 0x00  ........


#ifdef LCD
.org 512

; Zeile 0 (256 Zeichen):
.db 0x00,0x7E,0x7E,0x6C,0x10,0x38,0x10,0x10,0xFF,0x00,0x00,0x00,0x00,0x00,0x7F,0x99,0x80,0x02,0x18,0x66,0x7F,0x3E,0x00,0x18,0x18,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x6C,0x6C,0x30,0x00,0x38,0x60,0x18,0x60,0x00,0x00,0x00,0x00,0x00,0x06,0x7C,0x30,0x78,0x78,0x1C,0xFC,0x38,0xFC,0x78,0x78,0x00,0x00,0x18,0x00,0x60,0x78 ;1
.db 0x7C,0x30,0xFC,0x3C,0xF8,0xFE,0xFE,0x3C,0xCC,0x78,0x1E,0xE6,0xF0,0xC6,0xC6,0x38,0xFC,0x78,0xFC,0x78,0xFC,0xCC,0xCC,0xC6,0xC6,0xCC,0xFE,0x78,0xC0,0x78,0x10,0x00,0x30,0x00,0xE0,0x00,0x1C,0x00,0x38,0x00,0xE0,0x30,0x0C,0xE0,0x70,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x1C,0x18,0xE0,0x76,0x00 
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0x7E,0xCC,0xE0,0xCC,0x7C,0xE0,0xC6,0x30,0x1C,0x00,0x3E,0x78,0x00,0x00,0x78,0x00,0x00,0xC3,0xCC,0x18,0x38,0xCC,0xF8,0x0E,0x1C,0x38,0x00,0x00,0x00,0xFC,0x3C,0x38,0x30,0x00,0x00,0xC3,0xC3,0x18,0x00,0x00,0x22,0x55,0xDB,0x18,0x18,0x18,0x36,0x00,0x00,0x36,0x36,0x00,0x36,0x36,0x18,0x00
.db 0x18,0x18,0x00,0x18,0x00,0x18,0x18,0x36,0x36,0x00,0x36,0x00,0x36,0x00,0x36,0x18,0x36,0x00,0x00,0x36,0x18,0x00,0x00,0x36,0x18,0x18,0x00,0xFF,0x00,0xF0,0x0F,0xFF,0x00,0x00,0x00,0x00,0xFC,0x00,0x00,0x00,0xFC,0x38,0x38,0x1C,0x00,0x06,0x38,0x78,0x00,0x30,0x60,0x18,0x0E,0x18,0x30,0x00,0x38,0x00,0x00,0x0F,0x78,0x70,0x00,0x00 

; Zeile 1 (256 Zeichen):
.db 0x00,0x81,0xFF,0xFE,0x38,0x7C,0x10,0x10,0xFF,0x00,0x00,0x00,0x00,0x00,0x63,0x5A,0xE0,0x0E,0x3C,0x66,0xDB,0x63,0x00,0x3C,0x3C,0x18,0x18,0x30,0x30,0x30,0x30,0x30,0x00,0x78,0x6C,0x6C,0x7C,0xC6,0x6C,0x60,0x30,0x30,0x66,0x30,0x00,0x00,0x00,0x0C,0xC6,0x70,0xCC,0xCC,0x3C,0xC0,0x60,0xCC,0xCC,0xCC,0x30,0x30,0x30,0x00,0x30,0xCC ;2
.db 0xC6,0x78,0x66,0x66,0x6C,0x62,0x62,0x66,0xCC,0x30,0x0C,0x66,0x60,0xEE,0xE6,0x6C,0x66,0xCC,0x66,0xCC,0xB4,0xCC,0xCC,0xC6,0xC6,0xCC,0xC6,0x60,0x60,0x18,0x38,0x00,0x30,0x00,0x60,0x00,0x0C,0x00,0x6C,0x00,0x60,0x00,0x00,0x60,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x18,0x30,0xDC,0x10 
.db 0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0xFF,0xC3,0x00,0x00,0x00,0xC6,0x00,0x38,0x30,0x00,0x00,0x6C,0xCC,0xCC,0xE0,0xCC,0xE0,0xCC,0x18,0x00,0x18,0x6C,0xCC,0xCC,0x1B,0x00,0x00,0x1C,0x1C,0xF8,0x00,0x6C,0x6C,0x00,0x00,0x00,0xC6,0xC6,0x18,0x33,0xCC,0x88,0xAA,0x77,0x18,0x18,0x18,0x36,0x00,0x00,0x36,0x36,0x00,0x36,0x36,0x18,0x00
.db 0x18,0x18,0x00,0x18,0x00,0x18,0x18,0x36,0x36,0x00,0x36,0x00,0x36,0x00,0x36,0x18,0x36,0x00,0x00,0x36,0x18,0x00,0x00,0x36,0x18,0x18,0x00,0xFF,0x00,0xF0,0x0F,0xFF,0x00,0x78,0xFC,0xFE,0xCC,0x00,0x66,0x76,0x30,0x6C,0x6C,0x30,0x00,0x0C,0x60,0xCC,0xFC,0x30,0x30,0x30,0x1B,0x18,0x30,0x76,0x6C,0x00,0x00,0x0C,0x6C,0x18,0x00,0x00 

; Zeile 2 (256 Zeichen):
.db 0x00,0xA5,0xDB,0xFE,0x7C,0x38,0x38,0x38,0xE7,0x00,0x00,0x00,0x00,0x00,0x7F,0x3C,0xF8,0x3E,0x7E,0x66,0xDB,0x38,0x00,0x7E,0x7E,0x18,0x0C,0x60,0x60,0x60,0x60,0x60,0x00,0x78,0x6C,0xFE,0xC0,0xCC,0x38,0xC0,0x60,0x18,0x3C,0x30,0x00,0x00,0x00,0x18,0xCE,0x30,0x0C,0x0C,0x6C,0xF8,0xC0,0x0C,0xCC,0xCC,0x30,0x30,0x60,0xFC,0x18,0x0C ;3
.db 0xDE,0xCC,0x66,0xC0,0x66,0x68,0x68,0xC0,0xCC,0x30,0x0C,0x6C,0x60,0xFE,0xF6,0xC6,0x66,0xCC,0x66,0xE0,0x30,0xCC,0xCC,0xC6,0x6C,0xCC,0x8C,0x60,0x30,0x18,0x6C,0x00,0x18,0x78,0x60,0x78,0x0C,0x78,0x60,0x76,0x6C,0x70,0x0C,0x66,0x30,0xCC,0xF8,0x78,0xDC,0x76,0xDC,0x7C,0x7C,0xCC,0xCC,0xC6,0xC6,0xCC,0xFC,0x30,0x18,0x30,0x00,0x38 
.db 0x00,0x00,0x00,0x00,0x00,0xFF,0xFF,0xFF,0x3C,0x78,0x78,0x70,0x38,0x70,0x6C,0x00,0xFC,0x7F,0xCC,0x00,0x00,0x00,0x00,0x00,0x00,0x3C,0xCC,0x7E,0x64,0x78,0xCC,0x18,0x78,0x70,0x00,0x00,0x00,0xCC,0x6C,0x6C,0x30,0x00,0x00,0xCC,0xCC,0x00,0x66,0x66,0x22,0x55,0xDB,0x18,0x18,0xF8,0x36,0x00,0xF8,0xF6,0x36,0xFE,0xF6,0x36,0xF8,0x00
.db 0x18,0x18,0x00,0x18,0x00,0x18,0x1F,0x36,0x37,0x3F,0xF7,0xFF,0x37,0xFF,0xF7,0xFF,0x36,0xFF,0x00,0x36,0x1F,0x1F,0x00,0x36,0xFF,0x18,0x00,0xFF,0x00,0xF0,0x0F,0xFF,0x76,0xCC,0xCC,0x6C,0x60,0x7E,0x66,0xDC,0x78,0xC6,0xC6,0x18,0x7E,0x7E,0xC0,0xCC,0x00,0xFC,0x18,0x60,0x1B,0x18,0x00,0xDC,0x6C,0x00,0x00,0x0C,0x6C,0x30,0x3C,0x00 

; Zeile 3 (256 Zeichen):
.db 0x00,0x81,0xFF,0xFE,0xFE,0xFE,0x7C,0x7C,0xC3,0x00,0x00,0x00,0x00,0x00,0x63,0xE7,0xFE,0xFE,0x18,0x66,0x7B,0x6C,0x00,0x18,0x18,0x18,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0x00,0x30,0x00,0x6C,0x78,0x18,0x76,0x00,0x60,0x18,0xFF,0xFC,0x00,0xFC,0x00,0x30,0xDE,0x30,0x38,0x38,0xCC,0x0C,0xF8,0x18,0x78,0x7C,0x00,0x00,0xC0,0x00,0x0C,0x18 ;4
.db 0xDE,0xCC,0x7C,0xC0,0x66,0x78,0x78,0xC0,0xFC,0x30,0x0C,0x78,0x60,0xFE,0xDE,0xC6,0x7C,0xCC,0x7C,0x70,0x30,0xCC,0xCC,0xD6,0x38,0x78,0x18,0x60,0x18,0x18,0xC6,0x00,0x00,0x0C,0x7C,0xCC,0x7C,0xCC,0xF0,0xCC,0x76,0x30,0x0C,0x6C,0x30,0xFE,0xCC,0xCC,0x66,0xCC,0x76,0xC0,0x30,0xCC,0xCC,0xD6,0x6C,0xCC,0x98,0xE0,0x00,0x1C,0x00,0x6C 
.db 0x00,0x00,0x00,0x00,0xFF,0xFF,0xFF,0xFF,0x66,0xCC,0xCC,0x30,0x18,0x30,0xC6,0x78,0x60,0x0C,0xFE,0x78,0x78,0x78,0xCC,0xCC,0xCC,0x66,0xCC,0xC0,0xF0,0xFC,0xFA,0x3C,0x0C,0x30,0x78,0xCC,0xF8,0xEC,0x3E,0x38,0x60,0xFC,0xFC,0xDE,0xDB,0x18,0xCC,0x33,0x88,0xAA,0xEE,0x18,0x18,0x18,0x36,0x00,0x18,0x06,0x36,0x06,0x06,0x36,0x18,0x00
.db 0x18,0x18,0x00,0x18,0x00,0x18,0x18,0x36,0x30,0x30,0x00,0x00,0x30,0x00,0x00,0x00,0x36,0x00,0x00,0x36,0x18,0x18,0x00,0x36,0x18,0x18,0x00,0xFF,0x00,0xF0,0x0F,0xFF,0xDC,0xF8,0xC0,0x6C,0x30,0xD8,0x66,0x18,0xCC,0xFE,0xC6,0x7C,0xDB,0xDB,0xF8,0xCC,0xFC,0x30,0x30,0x30,0x18,0x18,0xFC,0x00,0x38,0x18,0x00,0x0C,0x6C,0x60,0x3C,0x00

; Zeile 4 (256 Zeichen):
.db 0x00,0xBD,0xC3,0x7C,0x7C,0xFE,0xFE,0xFE,0xC3,0x00,0x00,0x00,0x00,0x00,0x63,0xE7,0xF8,0x3E,0x18,0x66,0x1B,0x6C,0x7E,0x7E,0x18,0x7E,0x0C,0x60,0x60,0x60,0x60,0x60,0x00,0x30,0x00,0xFE,0x0C,0x30,0xDC,0x00,0x60,0x18,0x3C,0x30,0x00,0x00,0x00,0x60,0xF6,0x30,0x60,0x0C,0xFE,0x0C,0xCC,0x30,0xCC,0x0C,0x00,0x00,0x60,0x00,0x18,0x30 ;5
.db 0xDE,0xFC,0x66,0xC0,0x66,0x68,0x68,0xCE,0xCC,0x30,0xCC,0x6C,0x62,0xD6,0xCE,0xC6,0x60,0xDC,0x6C,0x1C,0x30,0xCC,0xCC,0xFE,0x38,0x30,0x32,0x60,0x0C,0x18,0x00,0x00,0x00,0x7C,0x66,0xC0,0xCC,0xFC,0x60,0xCC,0x66,0x30,0x0C,0x78,0x30,0xFE,0xCC,0xCC,0x66,0xCC,0x66,0x78,0x30,0xCC,0xCC,0xFE,0x38,0xCC,0x30,0x30,0x18,0x30,0x00,0xC6 
.db 0x00,0x00,0x00,0xFF,0xFF,0xFF,0xFF,0xFF,0x7E,0xFC,0xFC,0x30,0x18,0x30,0xFE,0xCC,0x78,0x7F,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0x66,0xCC,0xC0,0x60,0x30,0xC6,0x18,0x7C,0x30,0xCC,0xCC,0xCC,0xFC,0x00,0x00,0xC0,0xC0,0x0C,0x33,0x37,0x18,0x66,0x66,0x22,0x55,0xDB,0x18,0xF8,0xF8,0xF6,0xFE,0xF8,0xF6,0x36,0xF6,0xFE,0xFE,0xF8,0xF8
.db 0x1F,0xFF,0xFF,0x1F,0xFF,0xFF,0x1F,0x37,0x3F,0x37,0xFF,0xF7,0x37,0xFF,0xF7,0xFF,0xFF,0xFF,0xFF,0x3F,0x1F,0x1F,0x3F,0xFF,0xFF,0xF8,0x1F,0xFF,0xFF,0xF0,0x0F,0x00,0xC8,0xCC,0xC0,0x6C,0x60,0xD8,0x66,0x18,0xCC,0xC6,0x6C,0xCC,0xDB,0xDB,0xC0,0xCC,0x00,0x30,0x60,0x18,0x18,0x18,0x00,0x76,0x00,0x18,0x18,0xEC,0x6C,0x78,0x3C,0x00 

; Zeile 5 (256 Zeichen):
.db 0x00,0x99,0xE7,0x38,0x38,0x7C,0x7C,0x7C,0xE7,0x00,0x00,0x00,0x00,0x00,0x67,0x3C,0xE0,0x0E,0x7E,0x00,0x1B,0x38,0x7E,0x3C,0x18,0x3C,0x18,0x30,0x30,0x30,0x30,0x30,0x00,0x00,0x00,0x6C,0xF8,0x66,0xCC,0x00,0x30,0x30,0x66,0x30,0x30,0x00,0x30,0xC0,0xE6,0x30,0xCC,0xCC,0x0C,0xCC,0xCC,0x30,0xCC,0x18,0x30,0x30,0x30,0xFC,0x30,0x00 ;6
.db 0xC0,0xCC,0x66,0x66,0x6C,0x62,0x60,0x66,0xCC,0x30,0xCC,0x66,0x66,0xC6,0xC6,0x6C,0x60,0x78,0x66,0xCC,0x30,0xCC,0x78,0xEE,0x6C,0x30,0x66,0x60,0x06,0x18,0x00,0x00,0x00,0xCC,0x66,0xCC,0xCC,0xC0,0x60,0x7C,0x66,0x30,0xCC,0x6C,0x30,0xD6,0xCC,0xCC,0x7C,0x7C,0x60,0x0C,0x34,0xCC,0x78,0xFE,0x6C,0x7C,0x64,0x30,0x18,0x30,0x00,0xC6 
.db 0x00,0x00,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x60,0xC0,0xC0,0x30,0x18,0x30,0xC6,0xFC,0x60,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0x7C,0x3C,0xCC,0x7E,0xE6,0xFC,0xCF,0x18,0xCC,0x30,0xCC,0xCC,0xCC,0xDC,0x7E,0x7C,0xCC,0xC0,0x0C,0x66,0x6F,0x18,0x33,0xCC,0x88,0xAA,0x77,0x18,0x18,0x18,0x36,0x36,0x18,0x36,0x36,0x36,0x00,0x00,0x00,0x18
.db 0x00,0x00,0x18,0x18,0x00,0x18,0x18,0x36,0x00,0x36,0x00,0x36,0x36,0x00,0x36,0x00,0x00,0x18,0x36,0x00,0x00,0x18,0x36,0x36,0x18,0x00,0x18,0xFF,0xFF,0xF0,0x0F,0x00,0xDC,0xF8,0xC0,0x6C,0xCC,0xD8,0x7C,0x18,0x78,0x6C,0x6C,0xCC,0x7E,0x7E,0x60,0xCC,0xFC,0x00,0x00,0x00,0x18,0xD8,0x30,0xDC,0x00,0x00,0x00,0x6C,0x00,0x00,0x3C,0x00 

; Zeile 6 (256 Zeichen):
.db 0x00,0x81,0xFF,0x10,0x10,0x38,0x38,0x38,0xFF,0x00,0x00,0x00,0x00,0x00,0xE6,0x5A,0x80,0x02,0x3C,0x66,0x1B,0xCC,0x7E,0x18,0x18,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x00,0x6C,0x30,0xC6,0x76,0x00,0x18,0x60,0x00,0x00,0x30,0x00,0x30,0x80,0x7C,0xFC,0xFC,0x78,0x1E,0x78,0x78,0x30,0x78,0x70,0x30,0x30,0x18,0x00,0x60,0x30 ;7
.db 0x78,0xCC,0xFC,0x3C,0xF8,0xFE,0xF0,0x3E,0xCC,0x78,0x78,0xE6,0xFE,0xC6,0xC6,0x38,0xF0,0x1C,0xE6,0x78,0x78,0xFC,0x30,0xC6,0xC6,0x78,0xFE,0x78,0x02,0x78,0x00,0x00,0x00,0x76,0xDC,0x78,0x76,0x78,0xF0,0x0C,0xE6,0x78,0xCC,0xE6,0x78,0xC6,0xCC,0x78,0x60,0x0C,0xF0,0xF8,0x18,0x76,0x30,0x6C,0xC6,0x0C,0xFC,0x1C,0x18,0xE0,0x00,0xFE 
.db 0x00,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x3C,0x78,0x78,0x78,0x3C,0x78,0xC6,0xCC,0xFC,0x7F,0xCE,0x78,0x78,0x78,0x7E,0x7E,0x0C,0x18,0x78,0x18,0xFC,0x30,0xC6,0xD8,0x7E,0x78,0x78,0x7E,0xCC,0xCC,0x00,0x00,0x78,0x00,0x00,0xCC,0xCF,0x18,0x00,0x00,0x22,0x55,0xDB,0x18,0x18,0x18,0x36,0x36,0x18,0x36,0x36,0x36,0x00,0x00,0x00,0x18
.db 0x00,0x00,0x18,0x18,0x00,0x18,0x18,0x36,0x00,0x36,0x00,0x36,0x36,0x00,0x36,0x00,0x00,0x18,0x36,0x00,0x00,0x18,0x36,0x36,0x18,0x00,0x18,0xFF,0xFF,0xF0,0x0F,0x00,0x76,0xC0,0xC0,0x6C,0xFC,0x70,0x60,0x18,0x30,0x38,0xEE,0x78,0x00,0x60,0x38,0xCC,0x00,0xFC,0xFC,0xFC,0x18,0xD8,0x30,0x00,0x00,0x00,0x00,0x3C,0x00,0x00,0x00,0x00 

; Zeile 7 (256 Zeichen):
.db 0x00,0x7E,0x7E,0x00,0x00,0x7C,0x7C,0x7C,0xFF,0x00,0x00,0x00,0x00,0x00,0xC0,0x99,0x00,0x00,0x18,0x00,0x00,0x78,0x00,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x00,0x00,0x00,0x00 ;8
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF8,0x00,0x00,0x78,0x00,0x00,0x00,0x00,0x00,0xF0,0x1E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF8,0x00,0x00,0x00,0x00,0x00,0x00 
.db 0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF8,0x00,0x00,0x18,0x00,0x30,0xC7,0x70,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0F,0x03,0x00,0x00,0x00,0x88,0xAA,0xEE,0x18,0x18,0x18,0x36,0x36,0x18,0x36,0x36,0x36,0x00,0x00,0x00,0x18 ;16
.db 0x00,0x00,0x18,0x18,0x00,0x18,0x18,0x36,0x00,0x36,0x00,0x36,0x36,0x00,0x36,0x00,0x00,0x18,0x36,0x00,0x00,0x18,0x36,0x36,0x18,0x00,0x18,0xFF,0xFF,0xF0,0x0F,0x00,0x00,0xC0,0x00,0x00,0x00,0x00,0xC0,0x00,0xFC,0x00,0x00,0x00,0x00,0xC0,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0x70,0x00,0x00,0x00,0x00,0x00,0x1C,0x00,0x00,0x00,0x00 

#else ;LCD

.org 512
; Zeile 0 (256 Zeichen):
.db 0x00,0x7E,0x7E,0x6C,0x10,0x38,0x10,0x10,0xFF,0x00,0x00,0x00,0x00,0x00,0x7F,0x99,0x80,0x02,0x18,0x66,0x7F,0x3E,0x00,0x18,0x18,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x6C,0x6C,0x30,0x00,0x38,0x60,0x18,0x60,0x00,0x00,0x00,0x00,0x00,0x06,0x7C,0x30,0x78,0x78,0x1C,0xFC,0x38,0xFC,0x78,0x78,0x00,0x00,0x18,0x00,0x60,0x78 
.db 0x7C,0x30,0xFC,0x3C,0xF8,0xFE,0xFE,0x3C,0xCC,0x78,0x1E,0xE6,0xF0,0xC6,0xC6,0x38,0xFC,0x78,0xFC,0x78,0xFC,0xCC,0xCC,0xC6,0xC6,0xCC,0xFE,0x78,0xC0,0x78,0x10,0x00,0x30,0x00,0xE0,0x00,0x1C,0x00,0x38,0x00,0xE0,0x30,0x0C,0xE0,0x70,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x1C,0x18,0xE0,0x76,0x00 
.db 0x78,0x00,0x1C,0x7E,0xCC,0xE0,0x30,0x00,0x7E,0xCC,0xE0,0xCC,0x7C,0xE0,0xC6,0x30,0x1C,0x00,0x3E,0x78,0x00,0x00,0x78,0x00,0x00,0xC3,0xCC,0x18,0x38,0xCC,0xF8,0x0E,0x1C,0x38,0x00,0x00,0x00,0xFC,0x3C,0x38,0x30,0x00,0x00,0xC3,0xC3,0x18,0x00,0x00,0x22,0x55,0xDB,0x18,0x18,0x18,0x36,0x00,0x00,0x36,0x36,0x00,0x36,0x36,0x18,0x00 
.db 0x18,0x18,0x00,0x18,0x00,0x18,0x18,0x36,0x36,0x00,0x36,0x00,0x36,0x00,0x36,0x18,0x36,0x00,0x00,0x36,0x18,0x00,0x00,0x36,0x18,0x18,0x00,0xFF,0x00,0xF0,0x0F,0xFF,0x00,0x00,0x00,0x00,0xFC,0x00,0x00,0x00,0xFC,0x38,0x38,0x1C,0x00,0x06,0x38,0x78,0x00,0x30,0x60,0x18,0x0E,0x18,0x30,0x00,0x38,0x00,0x00,0x0F,0x78,0x70,0x00,0x00 

; Zeile 1 (256 Zeichen):
.db 0x00,0x81,0xFF,0xFE,0x38,0x7C,0x10,0x10,0xFF,0x00,0x00,0x00,0x00,0x00,0x63,0x5A,0xE0,0x0E,0x3C,0x66,0xDB,0x63,0x00,0x3C,0x3C,0x18,0x18,0x30,0x30,0x30,0x30,0x30,0x00,0x78,0x6C,0x6C,0x7C,0xC6,0x6C,0x60,0x30,0x30,0x66,0x30,0x00,0x00,0x00,0x0C,0xC6,0x70,0xCC,0xCC,0x3C,0xC0,0x60,0xCC,0xCC,0xCC,0x30,0x30,0x30,0x00,0x30,0xCC 
.db 0xC6,0x78,0x66,0x66,0x6C,0x62,0x62,0x66,0xCC,0x30,0x0C,0x66,0x60,0xEE,0xE6,0x6C,0x66,0xCC,0x66,0xCC,0xB4,0xCC,0xCC,0xC6,0xC6,0xCC,0xC6,0x60,0x60,0x18,0x38,0x00,0x30,0x00,0x60,0x00,0x0C,0x00,0x6C,0x00,0x60,0x00,0x00,0x60,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x18,0x30,0xDC,0x10 
.db 0xCC,0xCC,0x00,0xC3,0x00,0x00,0x30,0x00,0xC3,0x00,0x00,0x00,0xC6,0x00,0x38,0x30,0x00,0x00,0x6C,0xCC,0xCC,0xE0,0xCC,0xE0,0xCC,0x18,0x00,0x18,0x6C,0xCC,0xCC,0x1B,0x00,0x00,0x1C,0x1C,0xF8,0x00,0x6C,0x6C,0x00,0x00,0x00,0xC6,0xC6,0x18,0x33,0xCC,0x88,0xAA,0x77,0x18,0x18,0x18,0x36,0x00,0x00,0x36,0x36,0x00,0x36,0x36,0x18,0x00 
.db 0x18,0x18,0x00,0x18,0x00,0x18,0x18,0x36,0x36,0x00,0x36,0x00,0x36,0x00,0x36,0x18,0x36,0x00,0x00,0x36,0x18,0x00,0x00,0x36,0x18,0x18,0x00,0xFF,0x00,0xF0,0x0F,0xFF,0x00,0x78,0xFC,0xFE,0xCC,0x00,0x66,0x76,0x30,0x6C,0x6C,0x30,0x00,0x0C,0x60,0xCC,0xFC,0x30,0x30,0x30,0x1B,0x18,0x30,0x76,0x6C,0x00,0x00,0x0C,0x6C,0x18,0x00,0x00 

; Zeile 2 (256 Zeichen):
.db 0x00,0xA5,0xDB,0xFE,0x7C,0x38,0x38,0x38,0xE7,0x00,0x00,0x00,0x00,0x00,0x7F,0x3C,0xF8,0x3E,0x7E,0x66,0xDB,0x38,0x00,0x7E,0x7E,0x18,0x0C,0x60,0x60,0x60,0x60,0x60,0x00,0x78,0x6C,0xFE,0xC0,0xCC,0x38,0xC0,0x60,0x18,0x3C,0x30,0x00,0x00,0x00,0x18,0xCE,0x30,0x0C,0x0C,0x6C,0xF8,0xC0,0x0C,0xCC,0xCC,0x30,0x30,0x60,0xFC,0x18,0x0C 
.db 0xDE,0xCC,0x66,0xC0,0x66,0x68,0x68,0xC0,0xCC,0x30,0x0C,0x6C,0x60,0xFE,0xF6,0xC6,0x66,0xCC,0x66,0xE0,0x30,0xCC,0xCC,0xC6,0x6C,0xCC,0x8C,0x60,0x30,0x18,0x6C,0x00,0x18,0x78,0x60,0x78,0x0C,0x78,0x60,0x76,0x6C,0x70,0x0C,0x66,0x30,0xCC,0xF8,0x78,0xDC,0x76,0xDC,0x7C,0x7C,0xCC,0xCC,0xC6,0xC6,0xCC,0xFC,0x30,0x18,0x30,0x00,0x38 
.db 0xC0,0x00,0x78,0x3C,0x78,0x78,0x78,0x78,0x3C,0x78,0x78,0x70,0x38,0x70,0x6C,0x00,0xFC,0x7F,0xCC,0x00,0x00,0x00,0x00,0x00,0x00,0x3C,0xCC,0x7E,0x64,0x78,0xCC,0x18,0x78,0x70,0x00,0x00,0x00,0xCC,0x6C,0x6C,0x30,0x00,0x00,0xCC,0xCC,0x00,0x66,0x66,0x22,0x55,0xDB,0x18,0x18,0xF8,0x36,0x00,0xF8,0xF6,0x36,0xFE,0xF6,0x36,0xF8,0x00 
.db 0x18,0x18,0x00,0x18,0x00,0x18,0x1F,0x36,0x37,0x3F,0xF7,0xFF,0x37,0xFF,0xF7,0xFF,0x36,0xFF,0x00,0x36,0x1F,0x1F,0x00,0x36,0xFF,0x18,0x00,0xFF,0x00,0xF0,0x0F,0xFF,0x76,0xCC,0xCC,0x6C,0x60,0x7E,0x66,0xDC,0x78,0xC6,0xC6,0x18,0x7E,0x7E,0xC0,0xCC,0x00,0xFC,0x18,0x60,0x1B,0x18,0x00,0xDC,0x6C,0x00,0x00,0x0C,0x6C,0x30,0x3C,0x00 

; Zeile 3 (256 Zeichen):
.db 0x00,0x81,0xFF,0xFE,0xFE,0xFE,0x7C,0x7C,0xC3,0x00,0x00,0x00,0x00,0x00,0x63,0xE7,0xFE,0xFE,0x18,0x66,0x7B,0x6C,0x00,0x18,0x18,0x18,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0x00,0x30,0x00,0x6C,0x78,0x18,0x76,0x00,0x60,0x18,0xFF,0xFC,0x00,0xFC,0x00,0x30,0xDE,0x30,0x38,0x38,0xCC,0x0C,0xF8,0x18,0x78,0x7C,0x00,0x00,0xC0,0x00,0x0C,0x18 
.db 0xDE,0xCC,0x7C,0xC0,0x66,0x78,0x78,0xC0,0xFC,0x30,0x0C,0x78,0x60,0xFE,0xDE,0xC6,0x7C,0xCC,0x7C,0x70,0x30,0xCC,0xCC,0xD6,0x38,0x78,0x18,0x60,0x18,0x18,0xC6,0x00,0x00,0x0C,0x7C,0xCC,0x7C,0xCC,0xF0,0xCC,0x76,0x30,0x0C,0x6C,0x30,0xFE,0xCC,0xCC,0x66,0xCC,0x76,0xC0,0x30,0xCC,0xCC,0xD6,0x6C,0xCC,0x98,0xE0,0x00,0x1C,0x00,0x6C 
.db 0xCC,0xCC,0xCC,0x06,0x0C,0x0C,0x0C,0xC0,0x66,0xCC,0xCC,0x30,0x18,0x30,0xC6,0x78,0x60,0x0C,0xFE,0x78,0x78,0x78,0xCC,0xCC,0xCC,0x66,0xCC,0xC0,0xF0,0xFC,0xFA,0x3C,0x0C,0x30,0x78,0xCC,0xF8,0xEC,0x3E,0x38,0x60,0xFC,0xFC,0xDE,0xDB,0x18,0xCC,0x33,0x88,0xAA,0xEE,0x18,0x18,0x18,0x36,0x00,0x18,0x06,0x36,0x06,0x06,0x36,0x18,0x00 
.db 0x18,0x18,0x00,0x18,0x00,0x18,0x18,0x36,0x30,0x30,0x00,0x00,0x30,0x00,0x00,0x00,0x36,0x00,0x00,0x36,0x18,0x18,0x00,0x36,0x18,0x18,0x00,0xFF,0x00,0xF0,0x0F,0xFF,0xDC,0xF8,0xC0,0x6C,0x30,0xD8,0x66,0x18,0xCC,0xFE,0xC6,0x7C,0xDB,0xDB,0xF8,0xCC,0xFC,0x30,0x30,0x30,0x18,0x18,0xFC,0x00,0x38,0x18,0x00,0x0C,0x6C,0x60,0x3C,0x00 

; Zeile 4 (256 Zeichen):
.db 0x00,0xBD,0xC3,0x7C,0x7C,0xFE,0xFE,0xFE,0xC3,0x00,0x00,0x00,0x00,0x00,0x63,0xE7,0xF8,0x3E,0x18,0x66,0x1B,0x6C,0x7E,0x7E,0x18,0x7E,0x0C,0x60,0x60,0x60,0x60,0x60,0x00,0x30,0x00,0xFE,0x0C,0x30,0xDC,0x00,0x60,0x18,0x3C,0x30,0x00,0x00,0x00,0x60,0xF6,0x30,0x60,0x0C,0xFE,0x0C,0xCC,0x30,0xCC,0x0C,0x00,0x00,0x60,0x00,0x18,0x30 
.db 0xDE,0xFC,0x66,0xC0,0x66,0x68,0x68,0xCE,0xCC,0x30,0xCC,0x6C,0x62,0xD6,0xCE,0xC6,0x60,0xDC,0x6C,0x1C,0x30,0xCC,0xCC,0xFE,0x38,0x30,0x32,0x60,0x0C,0x18,0x00,0x00,0x00,0x7C,0x66,0xC0,0xCC,0xFC,0x60,0xCC,0x66,0x30,0x0C,0x78,0x30,0xFE,0xCC,0xCC,0x66,0xCC,0x66,0x78,0x30,0xCC,0xCC,0xFE,0x38,0xCC,0x30,0x30,0x18,0x30,0x00,0xC6 
.db 0x78,0xCC,0xFC,0x3E,0x7C,0x7C,0x7C,0xC0,0x7E,0xFC,0xFC,0x30,0x18,0x30,0xFE,0xCC,0x78,0x7F,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0x66,0xCC,0xC0,0x60,0x30,0xC6,0x18,0x7C,0x30,0xCC,0xCC,0xCC,0xFC,0x00,0x00,0xC0,0xC0,0x0C,0x33,0x37,0x18,0x66,0x66,0x22,0x55,0xDB,0x18,0xF8,0xF8,0xF6,0xFE,0xF8,0xF6,0x36,0xF6,0xFE,0xFE,0xF8,0xF8 
.db 0x1F,0xFF,0xFF,0x1F,0xFF,0xFF,0x1F,0x37,0x3F,0x37,0xFF,0xF7,0x37,0xFF,0xF7,0xFF,0xFF,0xFF,0xFF,0x3F,0x1F,0x1F,0x3F,0xFF,0xFF,0xF8,0x1F,0xFF,0xFF,0xF0,0x0F,0x00,0xC8,0xCC,0xC0,0x6C,0x60,0xD8,0x66,0x18,0xCC,0xC6,0x6C,0xCC,0xDB,0xDB,0xC0,0xCC,0x00,0x30,0x60,0x18,0x18,0x18,0x00,0x76,0x00,0x18,0x18,0xEC,0x6C,0x78,0x3C,0x00 

; Zeile 5 (256 Zeichen):
.db 0x00,0x99,0xE7,0x38,0x38,0x7C,0x7C,0x7C,0xE7,0x00,0x00,0x00,0x00,0x00,0x67,0x3C,0xE0,0x0E,0x7E,0x00,0x1B,0x38,0x7E,0x3C,0x18,0x3C,0x18,0x30,0x30,0x30,0x30,0x30,0x00,0x00,0x00,0x6C,0xF8,0x66,0xCC,0x00,0x30,0x30,0x66,0x30,0x30,0x00,0x30,0xC0,0xE6,0x30,0xCC,0xCC,0x0C,0xCC,0xCC,0x30,0xCC,0x18,0x30,0x30,0x30,0xFC,0x30,0x00 
.db 0xC0,0xCC,0x66,0x66,0x6C,0x62,0x60,0x66,0xCC,0x30,0xCC,0x66,0x66,0xC6,0xC6,0x6C,0x60,0x78,0x66,0xCC,0x30,0xCC,0x78,0xEE,0x6C,0x30,0x66,0x60,0x06,0x18,0x00,0x00,0x00,0xCC,0x66,0xCC,0xCC,0xC0,0x60,0x7C,0x66,0x30,0xCC,0x6C,0x30,0xD6,0xCC,0xCC,0x7C,0x7C,0x60,0x0C,0x34,0xCC,0x78,0xFE,0x6C,0x7C,0x64,0x30,0x18,0x30,0x00,0xC6 
.db 0x18,0xCC,0xC0,0x66,0xCC,0xCC,0xCC,0x78,0x60,0xC0,0xC0,0x30,0x18,0x30,0xC6,0xFC,0x60,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0x7C,0x3C,0xCC,0x7E,0xE6,0xFC,0xCF,0x18,0xCC,0x30,0xCC,0xCC,0xCC,0xDC,0x7E,0x7C,0xCC,0xC0,0x0C,0x66,0x6F,0x18,0x33,0xCC,0x88,0xAA,0x77,0x18,0x18,0x18,0x36,0x36,0x18,0x36,0x36,0x36,0x00,0x00,0x00,0x18 
.db 0x00,0x00,0x18,0x18,0x00,0x18,0x18,0x36,0x00,0x36,0x00,0x36,0x36,0x00,0x36,0x00,0x00,0x18,0x36,0x00,0x00,0x18,0x36,0x36,0x18,0x00,0x18,0xFF,0xFF,0xF0,0x0F,0x00,0xDC,0xF8,0xC0,0x6C,0xCC,0xD8,0x7C,0x18,0x78,0x6C,0x6C,0xCC,0x7E,0x7E,0x60,0xCC,0xFC,0x00,0x00,0x00,0x18,0xD8,0x30,0xDC,0x00,0x00,0x00,0x6C,0x00,0x00,0x3C,0x00 

; Zeile 6 (256 Zeichen):
.db 0x00,0x81,0xFF,0x10,0x10,0x38,0x38,0x38,0xFF,0x00,0x00,0x00,0x00,0x00,0xE6,0x5A,0x80,0x02,0x3C,0x66,0x1B,0xCC,0x7E,0x18,0x18,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x00,0x6C,0x30,0xC6,0x76,0x00,0x18,0x60,0x00,0x00,0x30,0x00,0x30,0x80,0x7C,0xFC,0xFC,0x78,0x1E,0x78,0x78,0x30,0x78,0x70,0x30,0x30,0x18,0x00,0x60,0x30 
.db 0x78,0xCC,0xFC,0x3C,0xF8,0xFE,0xF0,0x3E,0xCC,0x78,0x78,0xE6,0xFE,0xC6,0xC6,0x38,0xF0,0x1C,0xE6,0x78,0x78,0xFC,0x30,0xC6,0xC6,0x78,0xFE,0x78,0x02,0x78,0x00,0x00,0x00,0x76,0xDC,0x78,0x76,0x78,0xF0,0x0C,0xE6,0x78,0xCC,0xE6,0x78,0xC6,0xCC,0x78,0x60,0x0C,0xF0,0xF8,0x18,0x76,0x30,0x6C,0xC6,0x0C,0xFC,0x1C,0x18,0xE0,0x00,0xFE 
.db 0x0C,0x7E,0x78,0x3F,0x7E,0x7E,0x7E,0x0C,0x3C,0x78,0x78,0x78,0x3C,0x78,0xC6,0xCC,0xFC,0x7F,0xCE,0x78,0x78,0x78,0x7E,0x7E,0x0C,0x18,0x78,0x18,0xFC,0x30,0xC6,0xD8,0x7E,0x78,0x78,0x7E,0xCC,0xCC,0x00,0x00,0x78,0x00,0x00,0xCC,0xCF,0x18,0x00,0x00,0x22,0x55,0xDB,0x18,0x18,0x18,0x36,0x36,0x18,0x36,0x36,0x36,0x00,0x00,0x00,0x18 
.db 0x00,0x00,0x18,0x18,0x00,0x18,0x18,0x36,0x00,0x36,0x00,0x36,0x36,0x00,0x36,0x00,0x00,0x18,0x36,0x00,0x00,0x18,0x36,0x36,0x18,0x00,0x18,0xFF,0xFF,0xF0,0x0F,0x00,0x76,0xC0,0xC0,0x6C,0xFC,0x70,0x60,0x18,0x30,0x38,0xEE,0x78,0x00,0x60,0x38,0xCC,0x00,0xFC,0xFC,0xFC,0x18,0xD8,0x30,0x00,0x00,0x00,0x00,0x3C,0x00,0x00,0x00,0x00 

; Zeile 7 (256 Zeichen):
.db 0x00,0x7E,0x7E,0x00,0x00,0x7C,0x7C,0x7C,0xFF,0x00,0x00,0x00,0x00,0x00,0xC0,0x99,0x00,0x00,0x18,0x00,0x00,0x78,0x00,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x00,0x00,0x00,0x00 
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF8,0x00,0x00,0x78,0x00,0x00,0x00,0x00,0x00,0xF0,0x1E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF8,0x00,0x00,0x00,0x00,0x00,0x00 
.db 0x78,0x00,0x00,0x00,0x00,0x00,0x00,0x38,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF8,0x00,0x00,0x18,0x00,0x30,0xC7,0x70,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0F,0x03,0x00,0x00,0x00,0x88,0xAA,0xEE,0x18,0x18,0x18,0x36,0x36,0x18,0x36,0x36,0x36,0x00,0x00,0x00,0x18 
.db 0x00,0x00,0x18,0x18,0x00,0x18,0x18,0x36,0x00,0x36,0x00,0x36,0x36,0x00,0x36,0x00,0x00,0x18,0x36,0x00,0x00,0x18,0x36,0x36,0x18,0x00,0x18,0xFF,0xFF,0xF0,0x0F,0x00,0x00,0xC0,0x00,0x00,0x00,0x00,0xC0,0x00,0xFC,0x00,0x00,0x00,0x00,0xC0,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0x70,0x00,0x00,0x00,0x00,0x00,0x1C,0x00,0x00,0x00,0x00 

#endif ;LCD

#endif ;gfxmode
